@@ -53,7 +53,7 @@ struct _FoobarWidgetClass
 {
 	GtkWindowClass panel_class;
 
-	void (*move_focus_out)  (FoobarWidget *foo);
+	gboolean (* popup_panel_menu) (FoobarWidget	*foobar);
 };
 
 GType		foobar_widget_get_type		(void) G_GNUC_CONST;
