@@ -66,43 +66,30 @@ void		add_menu_widget		(Menu *menu,
 void		set_menu_applet_orient	(Menu *menu,
 					 PanelOrient orient);
 
-void		setup_menuitem		(GtkWidget *menuitem,
-					 GtkWidget *pixmap,
+void		setup_menuitem		(GtkWidget  *menuitem,
+					 GtkWidget  *pixmap,
 					 const char *title);
 
-GtkWidget *	create_panel_root_menu	(PanelWidget *panel);
-GtkWidget *     create_panel_context_menu (PanelWidget *panel);
+GtkWidget      *create_panel_context_menu (PanelWidget *panel);
+GtkWidget      *create_panel_root_menu    (PanelWidget *panel);
 
 void		menu_properties		(Menu *menu);
 
-void		panel_lock		(GtkWidget *widget,
-					 gpointer data);
+void		panel_lock		(GtkWidget *widget);
 
 /*to be called on startup to load in some of the directories*/
 void		init_menus		(void);
 
-GtkWidget *	create_menu_at		(GtkWidget *menu,
-					 const char *menudir,
-					 gboolean launcher_add,
-					 const char *dir_name,
-					 const char *pixmap_name,
-					 gboolean fake_submenus,
-					 gboolean force);
-GtkWidget *	create_fake_menu_at	(const char *menudir,
-					 gboolean launcher_add,
-					 const char *dir_name,
-					 const char *pixmap_name);
-
-void		submenu_to_display	(GtkWidget *menuw, gpointer data);
 gboolean	menu_need_reread	(GtkWidget *menuw);
 
 void		setup_internal_applet_drag (GtkWidget *menuitem,
 					    const char *applet_type);
-GtkWidget *	create_root_menu	(GtkWidget *root_menu,
-					 gboolean fake_submenus,
-					 int flags,
-					 gboolean is_basep,
-					 gboolean extra_items);
+GtkWidget *	create_root_menu	(GtkWidget   *root_menu,
+					 PanelWidget *panel,
+					 gboolean     fake_submenus,
+					 int          flags,
+					 gboolean     is_basep,
+					 gboolean     extra_items);
 
 /* some gtk code cut-n-paste action */
 void		our_gtk_menu_position	(GtkMenu *menu);
@@ -122,10 +109,11 @@ enum {
 };
 
 GtkWidget *	panel_menu_new		(void);
-void		panel_load_menu_image_deferred (GtkWidget *image_menu_item,
+void		panel_load_menu_image_deferred (GtkWidget  *image_menu_item,
+						const char *stock_id,
 						const char *image_filename,
 						const char *fallback_image_filename,
-						gboolean force_image);
+						gboolean    force_image);
 /* Note, bind the following on 'show' or some such */
 void		panel_make_sure_menu_within_screen (GtkMenu *menu);
 
